#!/usr/bin/env python3
'''
    These are various tools used by mediacurator
'''

import subprocess
import os
import sys

# Import colorama for colored output
import colorama
colorama.init()

# Define color codes for colored output
cgreen = colorama.Fore.GREEN
cyellow = colorama.Fore.YELLOW
cred = colorama.Fore.RED
creset = colorama.Fore.RESET

def load_arguments():
    '''Get/load command parameters

    Returns:
        dict: A dictionary containing lists of options passed by the user
    '''
    arguments = {
        "directories": list(),
        "files": list(),
        "inputs": list(),
        "filters": list(),
        "outputs": list(),
        "printop": list(),
    }

    for arg in sys.argv:
        # Confirm with the user that they selected to delete found files
        if "-del" in arg:
            print(
                f"{cyellow}WARNING: Delete option selected!{creset}")
            if not user_confirm(f"Are you sure you wish to delete all found results after selected operations are successful? [Y/N] ?", color="yellow"):
                print(f"{cgreen}Exiting!{creset}")
                exit()
        elif "-in:" in arg:
            arguments["inputs"] += arg[4:].split(",")
        elif "-filters:" in arg:
            arguments["filters"] += arg[9:].split(",")
        elif "-out:" in arg:
            arguments["outputs"] += arg[5:].split(",")
        elif "-print:" in arg:
            arguments["printop"] += arg[7:].split(",")
        elif "-files:" in arg:
            arguments["files"] += arg[7:].split(",,")
        elif "-dirs:" in arg:
            arguments["directories"] += arg[6:].split(",,")

    return arguments


def detect_ffmpeg():
    '''Returns the version of ffmpeg that is installed or False

    Returns:
        str: The version number of the installed FFMPEG
        False: If version retrieval failed
    '''
    try:
        txt = subprocess.check_output(
            ['ffmpeg', '-version'], stderr=subprocess.STDOUT).decode()
        if "ffmpeg version" in txt:
            # Strip the useless text
            return txt.split(' ')[2]
    except:
        pass
    return False


def user_confirm(question, color=False):
    '''Returns the user's answer to a yes or no question

    Args:
        question (str): The user question
        color (str, optional): The preferred color for a question (red/yellow)
    Returns:
        bool: True for a positive response, False otherwise
    '''
    if color == "yellow":
        print(f"{cyellow}{question} {creset}", end='')
        answer = input()
    elif color == "red":
        print(f"{cred}{question} {creset}", end='')
        answer = input()
    else:
        answer = input(f"{question} ")
    if answer.lower() in ["y", "yes"]:
        return True
    elif answer.lower() in ["n", "no"]:
        return False
    print("Please answer with yes (Y) or no (N)...")
    return user_confirm(question)


def deletefile(filepath):
    '''Delete a file and return a boolean indicating success

    Args:
        filepath (str): The full filepath
    Returns:
        bool: True if successful, False otherwise
    '''
    try:
        os.remove(filepath)
    except OSError:
        print(f"{cred}Error deleting {filepath}{creset}")
        return False

    print(f"{cgreen}Successfully deleted {filepath}{creset}")
    return True


def findfreename(filepath, attempt=0):
    '''Find a free filename by appending [HEVC] or [HEVC](#) to the name if necessary

    Args:
        filepath (str): The full filepath
        attempt (int, optional): The number of attempts made
    Returns:
        str: The first free filepath found
    '''
    attempt += 1
    filename = str(filepath)[:str(filepath).rindex(".")]
    extension = str(filepath)[str(filepath).rindex("."):]
    hevcpath = filename + "[HEVC]" + extension
    copynumpath = filename + f"[HEVC]({attempt})" + extension

    if not os.path.exists(filepath) and attempt <= 2:
        return filepath
    elif not os.path.exists(hevcpath) and attempt <= 2:
        return hevcpath
    elif not os.path.exists(copynumpath):
        return copynumpath
    return findfreename(filepath, attempt)



#!/usr/bin/env python3
''' This module defines the MediaLibrary class, which manages information about the workspace and all videos in it.'''

from pathlib import Path

from .video import Video
from .tools import deletefile

# Import colorama for colored output
import colorama
colorama.init()

# Define color codes for colored output
cgreen = colorama.Fore.GREEN
creset = colorama.Fore.RESET

class MediaLibrary():
    '''This class manages information about the workspace and all videos in it.'''

    def __init__(self, files=False, directories=False, inputs=["any"], filters=[], verbose=False):
        '''
        Initializes a MediaLibrary instance with provided parameters.

        Args:
            files (list or False): A list of video files.
            directories (list or False): A list of directories containing videos directly or in subdirectories.
            inputs (list): A list of filters to keep when browsing the directories.
            filters (list): A list of filters to apply to the videos.
            verbose (bool): A flag to enable verbose output.
        '''

        if not files and not directories:
            return
        
        self.directories = None
        self.inputs = inputs
        self.filters = filters
        self.videos = dict()
        
        if files:
            for filepath in files:
                self.videos[filepath] = Video(filepath, verbose=verbose)

        if directories:
            self.directories = directories
            self.load_directories(verbose=verbose)
        
        self.filter_videos(verbose=verbose)

    def __str__(self):
        '''
        Returns a string representation of the MediaLibrary instance.

        Returns:
            str: Information about the MediaLibrary instance.
        '''

        text = ""
        if self.directories:
            text += f"MediaCurator is watching the following directories: "
            text += '\n    '.join(map(str, self.directories)) + '\n'
        text += f"MediaCurator is tracking {len(self.videos)} video files"
        return text

    def load_directories(self, verbose=False):
        '''
        Scans folders for video files respecting the inputs requested by the user and saves them to the videos dictionary.

        Args:
            verbose (bool): A flag to enable verbose output.
        '''

        print(f"{cgreen}Scanning files in {', '.join(map(str, self.directories))} for videos{creset}")
        videolist = []
        
        for directory in self.directories:
            path = Path(directory)
            # get all video filetypes
            if "wmv" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[wW][mM][vV]"))
            if "avi" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[aA][vV][iI]"))
            if "mkv" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[mM][kK][vV]"))
            if "mp4" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[mM][pP]4"))
            if "m4v" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[mM]4[vV]"))
            if "flv" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[fF][lL][vV]"))
            if "mpg" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[mM][pP][gG]"))
            if "mpeg" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[mM][pP][eE][gG]"))
            if "vid" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[vV][iI][dD]"))
            if "vob" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[vV][oO][bB]"))
            if "divx" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[dD][iI][vV][xX]"))
            if "ogm" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[oO][gG][mM]"))
            if "webm" in self.inputs or "any" in self.inputs or len(self.inputs) < 1:
                videolist += list(path.rglob("*.[wW][eE][bB][mM]"))
        
        # Remove folders
        videolist_tmp = videolist
        videolist = [video for video in videolist_tmp if video.is_file()]

        # Map it all to the videos dictionary as initiated Video objects
        print(f"{cgreen}Analyzing {len(videolist)} videos in {', '.join(map(str, self.directories))}{creset}")
        iteration = 0
        for video in videolist:
            if verbose:
                iteration += 1
                print(f'{int((iteration / len(videolist) * 100))}% complete', end='\r')
            
            self.videos[video] = Video(video, verbose=verbose)

    def filter_videos(self, verbose=False):
        '''
        Marks videos for operation in the videos dictionary (default is not to operate).

        Args:
            verbose (bool): A flag to enable verbose output.
        '''

        print(f"{cgreen}Filtering {len(self.videos)} videos for the requested parameters{creset}")

        for filepath in self.videos:

            # Filter for filetypes
            if len([filtr for filtr in self.filters if filtr in ["old", "mpeg4", "mpeg", "wmv3", "wmv", "h264", "hevc", "x265", "av1"]]) > 0:
                operate = False
                if "old" in self.filters and self.videos[filepath].codec not in ["hevc", "av1"]:
                    operate = True
                if ("mpeg4" in self.filters or "mpeg" in self.filters) and self.videos[filepath].codec in ["mpeg4", "msmpeg4v3"]:
                    operate = True
                if "mpeg" in self.filters and self.videos[filepath].codec in ["mpeg1video"]:
                    operate = True
                if ("wmv3" in self.filters or "wmv" in self.filters) and self.videos[filepath].codec in ["wmv3"]:
                    operate = True
                if "h264" in self.filters and self.videos[filepath].codec in ["h264"]:
                    operate = True
                if ("hevc" in self.filters or "x265" in self.filters) and self.videos[filepath].codec in ["hevc"]:
                    operate = True
                if "av1" in self.filters and self.videos[filepath].codec in ["av1"]:
                    operate = True
                self.videos[filepath].operate = operate

            # Keep video for operation if specified resolution
            if self.videos[filepath].operate and len([filtr for filtr in self.filters if filtr in ["lowres", "hd", "subsd", "sd", "720p", "1080p", "uhd"]]) > 0:
                operate = False

                if "subsd" in self.filters and self.videos[filepath].definition in ["subsd"]:
                    operate = True
                if "sd" in self.filters and self.videos[filepath].definition in ["sd"]:
                    operate = True
                if "720p" in self.filters and self.videos[filepath].definition in ["720p"]:
                    operate = True
                if "1080p" in self.filters and self.videos[filepath].definition in ["1080p"]:
                    operate = True
                if "uhd" in self.filters and self.videos[filepath].definition in ["uhd"]:
                    operate = True
                if "lowres" in self.filters and self.videos[filepath].definition in ["subsd", "sd"]:
                    operate = True
                if "hd" in self.filters and self.videos[filepath].definition in ["720p", "1080p", "uhd"]:
                    operate = True
                self.videos[filepath].operate = operate

            # Keep video for operation if ffmpeg error exists
            if self.videos[filepath].operate and len([filtr for filtr in self.filters if filtr in ["fferror"]]) > 0:
                operate = False
                if self.videos[filepath].error:
                    operate = True
                self.videos[filepath].operate = operate
            
        print(f"{cgreen}Found {len([filepath for filepath in self.videos if self.videos[filepath].operate])} videos for the requested parameters{creset}")

    def unwatch(self, filepath, delete=False):
        '''
        Removes a video from the index and deletes it if requested.

        Args:
            filepath (str): The full filepath of the video.
            delete (bool): If True, delete the file as well as removing it from the library.

        Returns:
            bool: True if operation successful, False otherwise.
        '''
        
        if delete:
            deletefile(filepath)
        try:
            video = self.videos.pop(filepath)
            if video:
                return video
        except KeyError:
            pass
        return False



#!/usr/bin/env python3
'''This module defines the Video class, which contains information and methods for video files.'''

from .tools import deletefile, findfreename
import subprocess
import os

# Import colorama for colored output
import colorama
colorama.init()

# Define color codes for colored output
cyellow = colorama.Fore.YELLOW
cred = colorama.Fore.RED
creset = colorama.Fore.RESET

class Video():
    '''Contains the information and methods of a video file.'''        

    def __init__(self, filepath, operate=True, verbose=False):
        '''Initializes a Video instance with provided parameters.

        Args:
            filepath (str): The full filepath of the video.
            operate (bool): A flag indicating if the video is to be operated on.
            verbose (bool): A flag indicating whether to enable verbose output.
        '''

        # Initialize attributes
        self.path = None
        self.filename_origin = None
        self.filesize = None
        self.filename_new = None
        self.filename_tmp = None
        self.operate = None
        self.codec = None
        self.error = None
        self.definition = None
        self.width = None
        self.height = None
        
        # Break down the full path into its components
        if os.name == 'nt':
            self.path = str(filepath)[:str(filepath).rindex("\\") + 1]
            self.filename_origin = str(filepath)[str(filepath).rindex("\\") + 1:]
        else:
            self.path = str(filepath)[:str(filepath).rindex("/") + 1]
            self.filename_origin = str(filepath)[str(filepath).rindex("/") + 1:]

        if not os.path.exists(filepath):
            self.error = f"FileNotFoundError: [Errno 2] No such file or directory: '{filepath}'"
            self.operate = operate
        else:
            # Mark the video for operation if specified manually by the user
            self.operate = operate

            # Gather information on the video
            self.filesize = self.detect_filesize(filepath)
            self.error = self.detect_fferror(filepath)
            self.codec = self.detect_codec(filepath)
            try:
                self.width, self.height = self.detect_resolution(filepath)
                self.definition = self.detect_definition(
                    width=self.width, 
                    height=self.height
                )
            except:
                self.width, self.height = False, False
                self.definition = False
        
        if self.error and verbose:
            print(f"{cred}There seems to be an error with \"{filepath}\"{creset}")
            print(f"{cred}    {self.error}{creset}")

    def __str__(self):
        '''Returns a short formatted string about the video.

        Returns:
            str: A short formatted string about the video.
        '''

        if type(self.error) is str and "FileNotFoundError" in self.error:
            return self.error
        
        text = f"{self.codec} - "

        # If the first character of the definition is not a number (e.g., UHD and not 720p), upper it
        if self.definition and self.definition[0] and not self.definition[0].isnumeric():
            text += f"{self.definition.upper()}: ({self.width}x{self.height}) - "
        else:
            text += f"{self.definition}: ({self.width}x{self.height}) - "

        # Return the size in MB or GB if more than 1024 MB
        if self.filesize >= 1024:
            text += f"{self.filesize / 1024 :.2f} GB - "
        else:
            text += f"{self.filesize} MB - "
        
        text += f"'{self.path + self.filename_origin}'"

        if self.error:
            text += f"{cred}\nErrors:{creset}"
            for err in self.error.splitlines():
                text += f"{cred}\n    {err}{creset}"

        return text

    __repr__ = __str__

    def fprint(self):
        '''Returns a long formatted string about the video.

        Returns:
            str: A long formatted string about the video.
        '''

        if type(self.error) is str and "FileNotFoundError" in self.error:
            return self.error
        
        text = f"{self.path + self.filename_origin}\n"

        if self.definition and self.definition[0] and not self.definition[0].isnumeric():
            text += f"    Definition:     {self.definition.upper()}: ({self.width}x{self.height})\n"
        else:
            text += f"    Definition:     {self.definition}: ({self.width}x{self.height})\n"

        text += f"    Codec:          {self.codec}\n"

        # Return the size in MB or GB if more than 1024 MB
        if self.filesize >= 1024:
            text += f"    Size:           {self.filesize / 1024 :.2f} GB"
        else:
            text += f"    Size:           {self.filesize} MB"

        if self.error:
            text += f"{cred}\n    Errors:{creset}"
            for err in self.error.splitlines():
                text += f"{cred}\n        {err}{creset}"

        return text

    def convert(self, vcodec="x265", acodec=False, extension="mkv", verbose=False):
        '''
        Converts the original file to the requested format / codec.

        Args:
            vcodec (str): The new video codec, supports av1 or x265.
            acodec (bool): Currently not enabled, will be for audio codecs.
            extension (str): The new video container format and file extension.
            verbose (bool): A flag enabling verbosity.

        Returns:
            bool: True if operation successful, False otherwise.
        '''

        # Setting new filename
        if "mp4" in extension:
            newfilename = self.filename_origin[:-4] + ".mp4"
            if os.path.exists(self.path + newfilename):
                newfilename = findfreename(self.path + newfilename)
                if os.name == 'nt':
                    newfilename = str(newfilename)[str(newfilename).rindex("\\") + 1:]
                else:
                    newfilename = str(newfilename)[str(newfilename).rindex("/") + 1:]
        else:
            newfilename = self.filename_origin[:-4] + ".mkv"
            if os.path.exists(self.path + newfilename):
                newfilename = findfreename(self.path + newfilename)
                if os.name == 'nt':
                    newfilename = str(newfilename)[str(newfilename).rindex("\\") + 1:]
                else:
                    newfilename = str(newfilename)[str(newfilename).rindex("/") + 1:]

        self.filename_tmp = newfilename

        # Setting ffmpeg
        args = ['ffmpeg', '-i', self.path + self.filename_origin]

        # Conversion options
        if vcodec == "av1":
            args += ['-c:v', 'libaom-av1', '-strict', 'experimental']
        elif vcodec == "x265" or vcodec == "hevc":
            args += ['-c:v', 'libx265']
            args += ['-max_muxing_queue_size', '1000']
        # Conversion output
        args += [self.path + self.filename_tmp]
        
        try:
            if verbose:
                subprocess.call(args)
            else:
                txt = subprocess.check_output(args, stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as e:
            deletefile(self.path + self.filename_tmp)
            self.filename_tmp = ""
            print(f"{cred}Conversion failed {e}{creset}")
            return False
        except KeyboardInterrupt:
            print(f"{cyellow}Conversion cancelled, cleaning up...{creset}")
            deletefile(self.path + self.filename_tmp)
            self.filename_tmp = ""
            exit()
        else:
            try:
                os.chmod(f"{self.path}{self.filename_tmp}", 0o777)
            except PermissionError:
                print(f"{cred}PermissionError on: '{self.path}{self.filename_tmp}'{creset}")
            self.filename_new = self.filename_tmp
            self.filename_tmp = ""
            return True

    @staticmethod
    def detect_fferror(filepath):
        '''Returns a string with the detected errors.

        Args:
            filepath (str): The full filepath of the video.

        Returns:
            str: The errors that have been found/happened.
            False: The lack of errors.
        '''
        try:
            args = ["ffprobe", "-v", "error", str(filepath)]
            output = subprocess.check_output(args, stderr=subprocess.STDOUT)
            output = output.decode().strip()
            if len(output) > 0:
                return output
        except (subprocess.CalledProcessError, IndexError):
            return f'{cred}There seems to be a "subprocess.CalledProcessError" error with \"{filepath}\"{creset}'
        return False

    @staticmethod
    def detect_codec(filepath):
        '''Returns a string with the detected codec.

        Args:
            filepath (str): The full filepath of the video.

        Returns:
            str: The codec that has been detected.
            False: An error in the codec fetching process.
        '''
        output = False
        try:
            args = ["ffprobe", "-v", "quiet", "-select_streams", "v:0", "-show_entries", "stream=codec_name", "-of", "default=noprint_wrappers=1:nokey=1", str(filepath)]
            output = subprocess.check_output(args, stderr=subprocess.STDOUT)
            # Decoding from binary, stripping whitespace, keep only last line
            # in case ffprobe added error messages over the requested information
            output = output.decode().strip()
        except (subprocess.CalledProcessError, IndexError):
            return False
        return output

    @staticmethod
    def detect_resolution(filepath):
        '''Returns a list with the detected width(0) and height(1).

        Args:
            filepath (str): The full filepath of the video.

        Returns:
            List: The detected width(0) and height(1).
            False: An error in the resolution fetching process.
        '''
        try:
            args = ["ffprobe", "-v", "quiet", "-select_streams", "v:0", "-show_entries", "stream=width,height", "-of", "csv=s=x:p=0", str(filepath)]
            output = subprocess.check_output(args, stderr=subprocess.STDOUT)
            # Decoding from binary, stripping whitespace, keep only last line
            # in case ffprobe added error messages over the requested information
            output = output.decode().strip()
            # See if we got convertible data
            output = [int(output.split("x")[0]), int(output.split("x")[1])]
        except (subprocess.CalledProcessError, IndexError):
            return False
        return output[0], output[1]

    @staticmethod
    def detect_definition(filepath=False, width=False, height=False):
        '''Returns a string with the detected definition corrected for dead space.

        Args:
            filepath (str): A string containing the full filepath.
            width (int): The width of the video.
            height (int): The height of the video.

        Returns:
            str: The classified definition in width(0) and height(1).
            False: An error in the process.
        '''
        if filepath:
            width, height = Video.detect_resolution(filepath)
        if not width and not height:
            return False
        
        if width >= 2160 or height >= 2160:
            return "uhd"
        elif width >= 1440 or height >= 1080:
            return "1080p"
        elif width >= 1280 or height >= 720:
            return "720p"
        elif height >= 480:
            return "sd"
        return "subsd"

    @staticmethod
    def detect_filesize(filepath):
        '''Returns an integer with size in MB.

        Args:
            filepath (str): A string containing the full filepath.

        Returns:
            int: The filesize in MB.
            False: An error in the process.
        '''
        try:
            size = int(os.path.getsize(filepath) / 1024 / 1024)
        except subprocess.CalledProcessError:
            return False
        return size



#!/usr/bin/env python3
'''
    MediaCurator is a Python command line tool to manage a media database.
        * List all the video's and their codecs with or without filters
        * Batch recode videos to more modern codecs (x265 / AV1) based on filters: extentions, codecs ...
    ex:
    mediacurator list -in:any -filters:old -dirs:/mnt/media/ >> ../medlist.txt
    mediacurator convert -del -in:any -filters:mpeg4 -out:x265,mkv -dirs:"/mnt/media/Movies/"
    mediacurator convert -del  -in:avi,mpg -dirs:/mnt/media/
'''

import sys

# Normal import
try:
    from mediacurator.library.video import Video
    from mediacurator.library.medialibrary import MediaLibrary
    from mediacurator.library.tools import detect_ffmpeg, user_confirm, load_arguments
# Allow local import for development purposes
except ModuleNotFoundError:
    from library.video import Video
    from library.medialibrary import MediaLibrary
    from library.tools import detect_ffmpeg, user_confirm, load_arguments

# Import colorama for colored output
import colorama
colorama.init()

# Define color codes for colored output
ccyan = colorama.Fore.CYAN
cblue = colorama.Fore.BLUE
cgreen = colorama.Fore.GREEN
cred = colorama.Fore.RED
creset = colorama.Fore.RESET

def main():
    '''
    MediaCurator's main function
    
    Returns:
    '''

    print(f"{colorama.Style.BRIGHT}")

    # confirm that the command has enough parameters
    if len(sys.argv) < 2:
        print(f"{cred}ERROR: Command not understood, please see documentation.{creset}")

    # confirm that ffmpeg in indeed installed
    ffmpeg_version = detect_ffmpeg()
    if not ffmpeg_version:
        print(f"{cred}No ffmpeg version detected{creset}")
        exit()
    print(f"{cblue}ffmpeg version detected: {ffmpeg_version}{creset}")

    # Get/load command parameters
    arguments = load_arguments()

    # Loading the media library
    if len(arguments["files"]) > 0:
        medialibrary = MediaLibrary(files = arguments["files"], inputs = arguments["inputs"], filters = arguments["filters"])
    elif len(arguments["directories"]) > 0:
        medialibrary = MediaLibrary(directories = arguments["directories"], inputs = arguments["inputs"], filters = arguments["filters"])
    else:
        print(f"{cred}ERROR: No files or directories selected.{creset}")
        return

    # Actions
    if sys.argv[1] == "list":
        # Pulling list of marked videos / original keys for the medialibrary.videos dictionary
        keylist = [filepath for filepath in medialibrary.videos if medialibrary.videos[filepath].operate]
        keylist.sort()

        for filepath in keylist:
            if medialibrary.videos[filepath].operate:
                if "formated" in arguments["printop"] or "verbose" in arguments["printop"]:
                    if medialibrary.videos[filepath].error:
                        print(f"{cred}{medialibrary.videos[filepath].fprint()}{creset}")
                    else:
                        print(medialibrary.videos[filepath].fprint())
                else:
                    if medialibrary.videos[filepath].error:
                        print(f"{cred}{medialibrary.videos[filepath]}{creset}")
                    else:
                        print(medialibrary.videos[filepath])

                # if marked for deletion delete and unwatch the video
                if "-del" in sys.argv:
                    medialibrary.unwatch(filepath, delete = True)
    elif sys.argv[1] == "test":
        # Pulling list of marked videos / original keys for the medialibrary.videos dictionary
        keylist = [filepath for filepath in medialibrary.videos if medialibrary.videos[filepath].operate]
        keylist.sort()

        for filepath in keylist:
            if medialibrary.videos[filepath].operate:
                if "formated" in arguments["printop"] or "verbose" in arguments["printop"]:
                    if medialibrary.videos[filepath].error:
                        print(f"{cred}{medialibrary.videos[filepath].fprint()}{creset}")
                    else:
                        print(medialibrary.videos[filepath].fprint())
                else:
                    if medialibrary.videos[filepath].error:
                        print(f"{cred}{medialibrary.videos[filepath]}{creset}")
                    else:
                        print(medialibrary.videos[filepath])

                # if marked for deletion delete and unwatch the video
                if "-del" in sys.argv:
                    medialibrary.unwatch(filepath, delete = True)
        
    elif sys.argv[1] == "convert":
        counter = 0

        # Pulling list of marked videos / original keys for the medialibrary.videos dictionary
        keylist = [filepath for filepath in medialibrary.videos if medialibrary.videos[filepath].operate]
        keylist.sort()

        for filepath in keylist:
            counter += 1
            # Setting required variables
            if "av1" in arguments["outputs"]:
                vcodec = "av1"
            else:
                vcodec = "x265"

            # Verbosing
            print(f"{cgreen}******  Starting conversion {counter} of {len(keylist)}: '{ccyan}{medialibrary.videos[filepath].filename_origin}{cgreen}' from {ccyan}{medialibrary.videos[filepath].codec}{cgreen} to {ccyan}{vcodec}{cgreen}...{creset}")
            print(f"{ccyan}Original file:{creset}")
            if "formated" in arguments["printop"] or "verbose" in arguments["printop"]:
                print(medialibrary.videos[filepath].fprint())
            else:
                print(medialibrary.videos[filepath])

            print(f"{cgreen}Converting please wait...{creset}", end="\r")

            # Converting
            if medialibrary.videos[filepath].convert(verbose = "verbose" in arguments["printop"]):
                # Mark the job as done
                medialibrary.videos[filepath].operate = False

                # Scan the new video
                newfpath = medialibrary.videos[filepath].path + medialibrary.videos[filepath].filename_new
                
                medialibrary.videos[newfpath] = Video(newfpath, verbose = "verbose" in arguments["printop"])

                # Verbose
                print(f"{cgreen}Successfully converted '{medialibrary.videos[filepath].filename_origin}'{ccyan}({medialibrary.videos[filepath].filesize}mb){cgreen} to '{medialibrary.videos[newfpath].filename_origin}'{ccyan}({medialibrary.videos[newfpath].filesize}mb){cgreen}, {ccyan}new file:{creset}")

                if "formated" in arguments["printop"] or "verbose" in arguments["printop"]:
                    if medialibrary.videos[newfpath].error:
                        print(f"{cred}{medialibrary.videos[newfpath].fprint()}{creset}")
                    else:
                        print(medialibrary.videos[newfpath].fprint())
                else:
                    if medialibrary.videos[newfpath].error:
                        print(f"{cred}{medialibrary.videos[newfpath]}{creset}")
                    else:
                        print(medialibrary.videos[newfpath])

                # if marked for deletion delete and unwatch the video
                if "-del" in sys.argv:
                    medialibrary.unwatch(filepath, delete = True)

if __name__ == '__main__':
    main()
